/*
 * Copyright (C) 2010-2013 jeanfi@gmail.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */
#include <locale.h>
#include <libintl.h>
#define _(str) gettext(str)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <curl/curl.h>

#include "http.h"

static int debug;

struct ucontent {
	char *data;
	size_t len;
};

static CURL *curl;

static size_t cbk_curl(void *buffer, size_t size, size_t nmemb, void *userp)
{
	size_t realsize;
	struct ucontent *mem;

	realsize = size * nmemb;
	mem = (struct ucontent *)userp;

	mem->data = realloc(mem->data, mem->len + realsize + 1);

	memcpy(&(mem->data[mem->len]), buffer, realsize);
	mem->len += realsize;
	mem->data[mem->len] = 0;

	return realsize;
}

void http_init()
{
	if (!curl)
		curl = curl_easy_init();
}

void http_cleanup()
{
	curl_easy_cleanup(curl);
}

char *http_get(const char *url, const char *content)
{
	struct ucontent chunk;

	chunk.data = malloc(1);
	chunk.len = 0;

	curl_easy_setopt(curl, CURLOPT_URL, url);
	curl_easy_setopt(curl, CURLOPT_VERBOSE, 0);
	if (content) {
		curl_easy_setopt(curl, CURLOPT_POSTFIELDS, content);
		curl_easy_setopt(curl,
				 CURLOPT_POSTFIELDSIZE,
				 (long)strlen(content));
	}
	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cbk_curl);
	curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&chunk);

	if (curl_easy_perform(curl) == CURLE_OK)
		return chunk.data;

	free(chunk.data);
	fprintf(stderr, _("HTTP request fail url=%s"), url);

	return NULL;
}

json_object *http_json_get(const char *url, struct json_object *j)
{
	const char *in;
	char *out;
	struct json_object *result;

	in = json_object_to_json_string(j);
	out = http_get(url, in);

	if (debug)
		printf("HTTP request= %s\n",
		       json_object_to_json_string(j));

	if (out) {
		result = json_tokener_parse(out);
		if (debug)
			printf("HTTP reply= %s\n", out);
		free(out);
		return result;
	}

	return NULL;
}
